;===============================================================================
;
;816NIX DIRECTORY STRUCTURE
;
;
;Directories are a filesystem's table of contents.   A directory entry  consists
;of an inode number followed by a filename, the latter which is  ordinary  text.
;The space occupied in a directory by a filename & its inode number is a "slot."
;A slot with $0000 in the inode number field represents an inactive &  therefore
;available slot.
;
;For efficiency reasons, the slot size is defined as 2^n,  where n is a non-zero
;positive integer.  The 816NIX filesystem uses n=5, resulting in a 32 byte  slot
;size.  Larger slot sizes permit longer filenames, at the expense  of  directory
;search speed.  The original UNIX S51K file system used a 16 byte directory slot
;size, thus allowing the use of 14 byte filenames.  Contemporaneous usage allows
;much larger filenames.  The size chosen for 816nix was a compromise between al-
;lowing the use of reasonably descriptive filenames while not  unduly  affecting
;directory search performance.
;
;Pictorially, the internal layout of a directory with the filenames "aaaaaaaaa",
;"bbbbbbbbb" & "ccccccccc" appears as follows :
;
;	             DIRECTORY LAYOUT
;	+=======================================+ $000000
;	| INUM | .                              | < this directory
;	++ $000020
;	| INUM | ..                             | < parent directory
;	++ $000040
;	| INUM | aaaaaaaaa                      |
;	++ $000060
;	| INUM | bbbbbbbbb                      |
;	++ $000080
;	| INUM | ccccccccc                      |
;	+=======================================+ $0000A0
;
;Each slot occupies 32 bytes, with the 16 bit  inode  number  (INUM)  associated
;with the filename occupying the 1st & 2nd bytes in the slot.   The remaining 30
;bytes are available for the filename.  If the filename is less than 30 bytes in
;length it will be padded with nulls.  The hex numbers to the right of the pict-
;ograph are offsets in the directory relative to its start.
;
;Directories are like ordinary files, in that they may opened & read by any app-
;lication that has suitable permissions.   However, only the kernel can write to
;a directory, which it will do when operations that create, move, rename,  link,
;unlink or delete a file are invoked.  As it is the inode that defines the loca-
;tion of a file, a file may  have  multiple  names or "links," all of which will
;point to the same inode.
;
;A search for a filename is a linear process: the kernel will read the directory
;a slot at a time & check for a filename match,  repeating the process until the
;filename has been found or the end of the directory has been reached.  The lin-
;ear search method is necessary because entries are not sorted into any particu-
;lar order.
;
;An effect of this organization is that the number of slots in a directory has a
;signficant effect on how quickly the kernel can locate a file when it is to  be
;accessed.  Like any other files, directories expand in size as more entries are
;made.  Once the number of slots in the directory exceeds 672,  indirect  blocks
;will have to be used to point to the rest of the directory.  The result is that
;the additional disk activity required to read both indirect &  data blocks will
;have a negative effect on directory search speed.
;
;
s_dsslot =1 << 5               ;slot size...
;
;	
;	The slot size must be 2^N,  where N is any positive integer.
;	For example:
;
;	    s_dsslot = 1 << 4
;
;	sets the slot size to 2^4 or 16 bytes.  N should not be less
;	than 4.
;	
;
s_filnam =s_dsslot-s_inum      ;filename field size
n_dcslot =s_lcblk/s_dsslot     ;slots per logical CDA block
n_ddslot =s_ldblk/s_dsslot     ;slots per logical HDA block
;
;
;	slot structural definitions...
;
ds_inum  =0                    ;inode number
ds_fnam  =ds_inum+s_inum       ;filename
;
	.end
