;===============================================================================
;
;816NIX FILESYSTEM
;
;The 816NIX filesystem models some aspects of the UNIX S51K filesystem, as well
;as the later Acer Fast Filesystem (AFS).  Like the S51K filesystem, the 816NIX
;filesystem uses a 1024 byte (1KB) logical block size, which on a hard disk con-
;sists of 2 contiguous 512 byte physical blocks.
;
;Pictorially, the layout of an 816NIX filesystem is as follows (read from bottom
;up):
;
;	     816NIX FILESYSTEM LAYOUT       LBO (note 4)
;	+=================================+ ====================
;	|         data block array        |
;	++ note 3    (fs_dba)
;	|           inode array           |
;	++ note 2    (fs_inode)
;	|   block allocation map (BAM)    |
;	++ note 1    (fs_bam)
;	|   inode allocation map (IAM)    |
;	++ $00000002 (fs_iam)
;	|      BAM tag logical block      |
;	++ $00000001 (fs_tag)
;	|           superblock            |
;	+=================================+ $00000000 (fs_super)
;	 1) Determined by size of IAM.
;	 2) Determined by size of BAM.
;	 3) Determined by size of inode array.
;	 4) Logical block offset in filesystem.
;
;Note in the above the references to "logical block offsets" (LBO) rather than
;a "logical block address" (LBA), which is the usual manner of addressing a disk
;drive.  The LBO is used to illustrate the relationships between the different
;parts of the filesystem.  When a filesystem is created actual LBAs will replace
;the LBOs shown above.
;
;As noted above, 1 KB logical blocks are used  any value is possible as long as
;the number of physical blocks in a logical block is equal to 2^n, where n is
;any non-negative integer.  Hence 2^0 means a logical block is 1 physical block,
;2^1 is 2 physical blocks, etc.  A 1 KB size lends itself well to better comput-
;ational efficiency in the kernel code that calculates disk addressing.  An even
;larger value would be more computationally efficient, but would be less effic-
;ient in utilizing disk space.
;
;The following discussion will elaborate on each of the above major elements.
;
; Superblock.  The superblock contains the administrative data needed by the
;  kernel to mount & manage the filesystem.  A corrupted superblock will render
;  the filesystem inaccessible.
;
; Inodes.  Also referred to as file descriptors, inodes encapsulate the admini-
;  strative data needed to access file contents.  Inodes pinpoint each file's
;  location on the storage device, its size, when the it was last accessed, & so
;  forth.  Inodes are identified by a number ranging from $000000 to $FFFFFF
;  (16,777,215).  An inode number in one filesystem has no relationship to the
;  same inode number in another filesystem.
;
; Data blocks.  The data blocks are where file data are stored.  In the case of
;  large files, indirect address blocks are used to track where the data blocks
;  proper are located.  This arrangement allows the kernel to use non-contiguous
;  storage in an efficient manner.  Each file's inode has a block address table
;  that points to the 1st 21 data blocks & up to 3 levels of indirection.
;
; Inode allocation map.  The IAM is a bitmap that indicates which inodes are in
;  use & which are free.  The 1st bit indicates the status of inode $00000001,
;  the 2nd, inode $00000002, the 3rd, inode $00000003, etc., with a set bit ind-
;  icating that its corresponding inode is available.  Inode number $00000000 is
;  not used but is assigned a bit in the bitmap.  When a new file is created the
;  kernel will scan the IAM to find the 1st available inode & allocate it to the
;  file.
;
;  Associated with the IAM is a "tag" bitmap.  Each set bit in the tag bitmap
;  indicates that the corresponding IAM block has at least 1 free inode.  If all
;  blocks controlled by that IAM block are used the IAM block's bit in the tag
;  bitmap will be cleared.  If an allocated inode is returned to the free inode
;  block pool the corresponding IAM block's tag bit will be set.  The tag bitmap
;  is part of the superblock.
;
;  The tag bitmap can index a maximum of 2048 IAM blocks, each of which controls
;  8192 inodes.  Therefore, the maximum mumber of inodes possible in a filesys-
;  tem is 2048  8192 or 16,777,216.  As previously noted, inode $000000 cannot
;  be assigned, which means the practical number of inodes is 16,777,215.
;
; Block allocation map.  The BAM is a bitmap that indicates which data blocks
;  are in use & which are free.  The 1st bit indicates the status of the 1st
;  data block, the 2nd bit, the 2nd data block, etc., with a set bit indicating
;  that the corresponding block is available.
;
;  Associated with the BAM is a "tag block," which is also a bitmap.  Each set
;  bit in the tag block indicates that the corresponding BAM block has at least
;  1 free data block.  If all blocks controlled by that BAM block are used the
;  BAM block's bit in the tag block will be cleared.  If an allocated data block
;  is returned to the free block pool the BAM block's tag bit will be set.
;
;  The tag block bitmap can index a maximum of 8192 BAM blocks, each of which
;  controls up to 8192 data blocks.  Hence the maximum number of data blocks
;  possible in a filesystem is 8192 or 67,108,864.  As a data block is atomic-
;  ally defined to be 1024 bytes in size, the absolute maximum size a filesys-
;  tem's data area can be is 1024  8192, which is 68,719,476,736 bytes (64 GB.
;
;  The tag block eliminates the need to sequentially scan the entire BAM to loc-
;  ate a free block. Instead, the kernel will scan the tag block to find the 1st
;  set bit & then go directly to the corresponding BAM block to locate a free
;  data block.  This procedure limits a worse-case search to two disk accesses.
;  If both the tag block & the correct BAM block are already in the buffer pool
;  then a disk access will be completely avoided.
;
;Note in the above filesystem pictograph that the space assigned to the super-
;block & BAM tag block fixed, as the sizes of these data structures are atomic-
;ally hard coded in the present implementation.  The size of IAM, BAM & inode
;array is determined by the number of inodes & data blocks in the filesystem.
;During filesystem creation, the system administrator determines the filesystem
;dimensions by entering the following values:
;
;	FSS     = filesystem size,  min = 2,107,395  max = 69,216,258
;	N_INODE = number of inodes, min =         8  max = 16,777,215
;
;Upon entry of the above values, the filesystem geometry is computed.  The fol-
;lowing constants are used:
;
;	S_LOGBLK = 1024  Size of a logical block in bytes.
;	IOH      = 2     Immutable filesystem overhead, consisting of the super-
;	                 block & BAM tag block.
;
;	S_INODE  = 128   Size of an inode in bytes.
;
;	N_IIBLK  = 8     Number of inodes per logical block.
;
;	r        = 8192  Bits in a logical block.
;	             
;Geometry computations are carried out as follows:
;
;	N_IAMBLK = N_INODE  r + (MOD(N_INODE,r) != 0)
;
;	N_IABLK	 = N_INODE  S_LOGBLK + (MOD(N_INODE,S_LOGBLK) != 0)
;
;	COH      = IOH + N_IAMBLK + N_IABLK
;
;	D        = (FSS  COH)  r  (r + 1)
;	B       = D  r + (MOD(D,r) != 0)
;	FSS     = FOH + D + D / r
;
;Terms are as follows:
;
;
;	N_IAM: Inode allocation map size in blocks, computed from the number of
;	       inodes configured into the filesystem.
;
;	N_INA: Inode array in blocks, computed from the number of inodes config-
;	       ured into the filesystem.
;
;	FOH  : Total filesystem overhead.
;where IOH is "immutable overhead,"  FOH is "filesystem overhead," which is the space consumed by the super-
;block (N_SB), IAM (N_IAM), inode array (N_INA) & BAM tag block, D is the number
;of data blocks & r is the number of bits in a logical block.  FSS is the total
;filesystem size.
;
;In the above equation, the only unknown is D.  FSS is set by the administrator
;when the filesystem is created, FOH is constant regardless of filesystem size &
;r may be empirically determined:
;
;	r = n  b
;
;As earlier explained, n is the number of physical blocks in a logical block &
;b is a constant equal to the number of bits in a physical block.  As a logical
;block consists of 2 physical blocks & a physical block is usually 512 bytes:
;
;	r = 2  512  8
;
;which is 8192 bits.
;   
;To determine D:
;
;	FSS - FOH = D + D  r
;
;For simplification purposes:
;
;	U = FSS - FOH
;
;making U the number of blocks available for the BAM & data.  As there is a def-
;ined relationship between the size of the BAM & the size of the data area, an
;expression may be written to describe it:
;
;	U = D + D  r
;
;Solving for D results in:
;
;	D = int(r  U  (r + 1))
;
;With D known, the required number of BAM blocks is:
;
;	B = int(D  r) + (mod(D,r) <> 0)
;
;The number of active bits in the BAM would be equal to D.
;
;
;DIRECTORY STRUCTURE
;
;Directories are a filesystem's table of contents.   A directory entry  consists
;of an inode number followed by a filename, the latter which is  ordinary  text.
;The space occupied in a directory by a filename & its inode number is a "slot."
;A slot with $00000000 in the inode number field represents an inactive & there-
;fore available slot.
;
;For efficiency reasons, the slot size is defined as 2^n,  where n is a non-zero
;positive integer.  The 816NIX filesystem uses n=6, resulting in a 64 byte  slot
;size.  Larger slot sizes permit longer filenames, at the expense  of  directory
;search speed.  The original UNIX S51K file system used a 16 byte directory slot
;size, thus allowing the use of 14 byte filenames.  Contemporaneous usage allows
;much larger filenames.  The size chosen for 816nix was a compromise between al-
;lowing the use of reasonably descriptive filenames while not  unduly  affecting
;directory search performance.
;
;Pictorially, the internal layout of a directory is as follows:
;
;	         DIRECTORY LAYOUT           Byte Offset in Directory
;	+=================================+ ========================
;	| INUM | ...etc...                |
;	++ s_dsslot*5
;	| INUM | <filename3>              | ...etc...
;	++ s_dsslot*4
;	| INUM | <filename2>              |
;	++ s_dsslot*3
;	| INUM | <filename1>              |
;	++ s_dsslot*2
;	| INUM | .. (parent)              |
;	++ s_dsslot
;	| INUM | .  (CWD)                 |
;	+=================================+ $00000000
;	 CWD: current working directory
;
;As previously noted, each slot occupies 64 bytes, with the 32 bit inode number
;(IN) associated with each filename occupying the 1st through 4th bytes in the
;slot.  The remaining 60 bytes are available for the filename.  If necessary,
;filenames are padded to 60 bytes with null bytes ($00).  A file may have mult-
;iple names or "links," all of which will point to the same inode.
;
;An effect of this organization is that the number of slots in a directory has a
;signficant effect on how quickly the kernel can locate a file when it is to  be
;accessed.  Like any other files, directories expand in size as more entries are
;made.   Once the number of slots in the directory exceeds 336  indirect  blocks
;will have to be used to point to the rest of the directory.  The result is that
;the additional disk activity required to read both indirect &  data blocks will
;have a negative effect on directory search speed.
;
;
;
;SUPERBLOCK STRUCTURE
;
;The superblock is the 1st block in a filesystem & contains the basic administr-
;ative information required for successful access.  Corruption of the superblock
;will generally render the filesystem inaccessible to the operating system.
;
;Pictorially, the superblock layout is as follows (read from bottom up):
;
;	         SUPERBLOCK LAYOUT          Byte Offset in Block
;	+=================================+ ====================
;	|        uncommitted space        |
;	++ $0150
;	|  IAM tag bit field (256 bytes)  |
;	++ $0050 (sb_iamta)
;	|    data storage starting LBA    |
;	++ $004C (sb_dbsa)
;	|        BAM starting LBA         |
;	++ $0048 (sb_bamsa)
;	|        BAM tag block LBA        |
;	++ $0044 (sb_bamta)
;	|      inode array start LBA      |
;	++ $0040 (sb_iasa)
;	|         IAM starting LBA        |
;	++ $003C (sb_iamsa)
;	|         free inode count        |
;	++ $0038 (sb_inodf)
;	|           total inodes          |
;	++ $0034 (sb_inod)
;	|      free data block count      |
;	++ $0030 (sb_dabf)
;	|        total data blocks        |
;	++ $002C (sb_dablk)
;	|        total inode blocks       |
;	++ $0028 (sb_inblk)
;	|         total BAM blocks        |
;	++ $0024 (sb_dmblk)
;	|         total IAM blocks        |
;	++ $0020 (sb_imblk)
;	|   logical blocks in filesystem  |
;	++ $001C (sb_lblks)
;	|  physical blocks in filesystem  |
;	++ $0018 (sb_pblks)
;	|     last modification time      |
;	++ $0012 (sb_mtime)
;	|       filesystem origin         |
;	++ $000E (sb_orig)
;	|          magic number           |
;	++ $000A (sb_magic)
;	|        filesystem name          |
;	++ $0002 (sb_fsnam)
;	|          status flag            |
;	++ $0001 (sb_mode)
;	|           lock flag             |
;	+=================================+ $0000 (sb_lock)
