;================================================================================
;
;getdtr: READ RTC DATE & TIME REGISTERS
;
;	
;	Preparatory Ops: .X: storage address LSB
;	                 .Y: storage address MSB
;
;	Returned Values: .A: entry value
;	                 .B: entry value
;	                 .X: entry value
;	                 .Y: entry value
;
;	                 Storage location will contain 8 BCD
;	                 bytes as follows:
;
;	                 Offset  Content      Range
;	                 --------------
;	                   $00   seconds      $00-$59
;	                   $01   minutes      $00-$59
;	                   $02   hours        $00-$23
;	                   $03   day-of-week  $01-$07
;	                   $04   date         $01-$31
;	                   $05   month        $01-$12
;	                   $06   year LSB     $00-$99
;	                   $07   year MSB     $00-$39
;	                 --------------
;
;	MPU Flags: NVmxDIZC
;	           ||||||||
;	           ++++++++> entry values
;
;	Example: ldx #<todbuf
;	         ldy #>todbuf
;	         jsr getdtr
;	
;
getdtr   php                   ;save MPU state
         longr
         phd
         phy
         phx
         pha
         lda !#kerneldp        ;set default...
         tcd                   ;direct page
         shortr
         tya                   ;storage address MSB
         xba                   ;move to MSB of .C
         txa                   ;storage address LSB
         longx                 ;16 bit index regs
         tax                   ;copy 16 bit storage index
         lda #d11temsk
         trb io_rtc+wr_crb     ;halt register updates
         ldy !#0               ;RTC register offset
;
.0000010 lda io_rtc,y          ;read RTC
         cpy !#wr_mon          ;month register?
         bne .0000020          ;no
;
         and #d11emmsk         ;discard control bits
;
.0000020 sta mm_ram,x          ;store in RAM
         cpy !#wr_yrhi-wr_sect ;all registers read?
         beq getdtraa          ;yes, done
;
         inx                   ;next location
         iny                   ;next register
         bra .0000010
;
getdtraa lda #d11temsk
         tsb io_rtc+wr_crb     ;resume register updates
         longr
         pla                   ;restore MPU entry state
         plx
         ply
         pld
         plp
         rts
;
;================================================================================
;
;getuptim: GET SYSTEM UP-TIME
;
;	
;	Preparatory Ops: .X: 16 bits: storage location
;
;	Returned Values: .A: entry value
;	                 .B: entry value
;	                 .X: entry value
;	                 .Y: entry value
;
;	MPU Flags: NVmxDIZC
;	           ||||||||
;	           ||||||++> entry value
;	           |||||+> 0
;	           +++++> entry value
;	
;
getsutim cli                   ;IRQs must be...
         php                   ;enabled on exit
         longr
         phy                   ;save MPU state
         phx
         pha
         lda !#uptime          ;data source
         ldy !#s_word          ;data index
         bra GETTMCOM          ;copy
;
;================================================================================
;
;getuxtim: GET UNIX TIME
;
;	
;	Preparatory Ops: .X: 16 bits: storage location
;
;	Returned Values: .A: entry value
;	                 .B: entry value
;	                 .X: entry value
;	                 .Y: entry value
;
;	MPU Flags: NVmxDIZC
;	           ||||||||
;	           ||||||++> entry value
;	           |||||+> 0
;	           +++++> entry value
;	
;
getuxtim cli                   ;IRQs must be...
         php                   ;enabled on exit
         longr
         phy                   ;save MPU state
         phx
         pha
         lda !#uxtime          ;data source
         ldy !#s_time_t-s_word ;data index
;
GETTMCOM pha                   ;set data source
         phd                   ;save entry DP
;
;
.reg_dp  =1                    ;DP
.source  =.reg_dp+s_word       ;data source (.C)
.reg_c   =.source+s_ptr        ;.C
.reg_x   =.reg_c+s_word        ;data destination
.reg_y   =.reg_x+s_word        ;.Y
.reg_sr  =.reg_y+s_word        ;SR
;
;
         lda !#kerneldp
         tcd                   ;set kernel direct page
         sei                   ;suspend updating
;
.0000010 lda (.source,s),y     ;copy source to...
         sta (.reg_x,s),y      ;destination
         decy s_word
         bpl .0000010          ;next
;
         pld                   ;clean up stack
         pla                   ;restore MPU state
         pla
         plx
         ply
         plp
         rts
;
;================================================================================
;
;putdtr: WRITE RTC DATE & TIME REGISTERS
;
;	
;	Preparatory Ops: .X: source address LSB
;	                 .Y: source address MSB
;
;	                 Source must contain 8 BCD values in
;	                 the following order:
;
;	                 Offset  Content      Range
;	                 --------------
;	                   $00   seconds      $00-$59
;	                   $01   minutes      $00-$59
;	                   $02   hours        $00-$23
;	                   $03   day-of-week  $01-$07
;	                   $04   date         $01-$31
;	                   $05   month        $01-$12
;	                   $06   year LSB     $00-$99
;	                   $07   year MSB     $00-$39
;	                 --------------
;
;	Returned Values: .A: entry value
;	                 .B: entry value
;	                 .X: entry value
;	                 .Y: entry value
;
;	MPU Flags: NVmxDIZC
;	           ||||||||
;	           ++++++++> entry values
;
;	Example: ldx #<todbuf
;	         ldy #>todbuf
;	         jsr putdtr
;	
;
putdtr   php                   ;save MPU state
         longr
         phd
         phy
         phx
         pha
         lda !#kerneldp        ;set default...
         tcd                   ;direct page
         shortr
         tya                   ;storage address MSB
         xba                   ;move to MSB of .C
         txa                   ;storage address LSB
         longx                 ;16 bit index regs
         tax                   ;source index
         lda #d11temsk
         trb io_rtc+wr_crb     ;halt register updates
         ldy !#0               ;register offset
;
.0000010 lda #0                ;dummy value
         cpy !#wr_mon          ;month register?
         bne .0000020          ;no
;
         lda io_rtc,y          ;get month & control
         and #d11ecmsk         ;extract control bits...
;
.0000020 ora mm_ram,x          ;get new value
         sta io_rtc,y          ;write RTC
         cpy !#wr_yrhi-wr_sect ;all registers written?
         beq getdtraa          ;yes, done
;
         inx                   ;next location
         iny                   ;next register
         bra .0000010
;
;================================================================================
;
;setuxtim: SET UNIX TIME
;
;	
;	Preparatory Ops: .Y: 16 bits: time number location
;
;	Returned Values: .A: used
;	                 .B: used
;	                 .X: used
;	                 .Y: used
;
;	MPU Flags: NVmxDIZC
;	           ||||||||
;	           ||||||++> entry value
;	           |||||+> 0
;	           +++++> entry value
;	
;
setuxtim cli                   ;IRQs must be enabled...
         php                   ;on exit
         longr
         phd
         phy
         phx
         pha
         lda !#kerneldp
         tcd                   ;set kernel direct page
         lda !#s_time_t-1      ;bytes to copy
         ldy !#uxtime          ;time "register"
         sei                   ;don't interrupt us...
         mvn #0,#0             ;while we copy
         pla
         plx
         ply
         pld
         plp
         rts
;
;================================================================================
;
;td2sec: GENERATE 2 SECOND TIME DELAY
;
td2sec   longa
         lda !#2
         shorta
;
;================================================================================
;
;utdelay: GENERATE USER-DEFINED TIME DELAY
;
;	
;	Preparatory Ops: .A: 16 bit delay time in secs
;
;	Returned Values: .A: entry value
;	                 .B: entry value
;	                 .X: entry value
;	                 .Y: entry value
;
;	MPU Flags: NVmxDIZC
;	           ||||||||
;	           ++++++++> entry values
;
;	Notes: 1) Delay time is approximate.
;	       2) A delay time of zero will cause an
;	          immediate exit.
;
;	Examples: longa           ;16 bit .A
;	          lda #600        ;600 secs
;	          jsr utdelay
;
;	          or...
;
;	          shorta          ;8 bit .A
;	          lda #>600       ;600 secs MSB in .A
;	          xba             ;transfer to .B
;	          lda #<600       ;600 secs LSB in .A
;	          jsr utdelay
;
;	NOTES: 1) Short time delay periods will not be
;	          very accurate.
;	
;
utdelay  php                   ;save MPU state
         longa
         ora !#0               ;zero delay?
         beq .0000020          ;yes, just exit
;
         phd
         pha                   ;save entry value
         pea #kerneldp
         pld                   ;set kernel direct page
         clc
         sei                   ;stop updates
         adc uptime            ;uptime LSW
         sta slptime           ;expire time LSW
         lda uptime+s_word     ;uptime MSW
         cli                   ;resume updates
         adc !#0               ;handle carry
         sta slptime+s_word    ;expire time MSW
         shorta
         lda #%10000000
         sta tdflag            ;initiate sleep period
;
.0000010 wai                   ;wait for next IRQ
         bit tdflag            ;sleep period expire?
         bmi .0000010          ;no
;
         longa                 ;yes, restore...
         pla                   ;entry value
         pld                   ;restore DP
;
.0000020 plp                   ;restore MPU state
         rts
;      
	.end