;================================================================================
;
;MACHINE LANGUAGE MONITOR  LOAD S-RECORD CODE
;
;	
;	syntax: L [<bank> [<offset>]]
;
;	<bank> is the 64KB RAM bank into which code is to be loaded.  If not
;	specified the bank value stored in the MPU's PB shadow register is used.
;	On completion of the load, the PB shadow register will be set to <bank>.
;
;	<offset> is an optional load address page offset.  This offset will be
;	added to each record's starting load address MSB to produce the effect-
;	ive load address.  Address wrap will occur if the sum of the load add-
;	ress MSB & page offset is greater than $FFthe bank address will not
;	wrap.  <bank> must be specified if <offset> is required.
;
;
;	Data Input Requirements
;	-----------
;	This function translates Motorola S-records into binary values that are
;	loaded into RAM.  If a non-zero page offset is specified, the load add-
;	ress information in each record will be modified by the offset value,
;	thus permitting a relocating load to a new starting page boundary.
;
;	The data stream is read from the auxiliary TIA-232 port until an S9 rec-
;	ord has been received & processed.  Pressing [Ctrl-C] will immediately
;	abort the process, will which result in a partial & most likely unusable
;	load.
;
;	The end of each record (EOR) is demarcated by an <LF> (UNIX newline) or
;	a <CR><LF> sequence (MS-DOS newline).  A missing EOR demarcation will
;	cause a buffer overrun error & prematurely halt the load.
;
;	The following S-record types are recognized:
;
;	    Type  Description
;	    ---------
;	      0   Block header
;	      1   Loadable data
;	      5   Record count
;	      9   Termination
;	    ---------
;
;	The present code dos not process the S0 record type in any way.
;
;	The general format of an S-record is as follows:
;
;	    <preamble> <type> <length> <address> <data> <checksum>
;
;	Excepting the preamble & type characters, data in each field is repre-
;	sented by pairs of ASCII characters that are textual equivalents of hex-
;	adecimal values.  For example, "1A" means the binary value $1A or deci-
;	mal 26.  The preamble & type characters taken together are used	to des-
;	cribe the record type, e.g., S0, S1, S5 or S9.
;
;	Field details are as follows:
;
;	    Field     Size  Description
;	    ----------------------------------
;	    preamble    1   a literal "S"
;
;	    type        1   record type, indicated by a literal ASCII numeral
;
;	    length      2   record length, equal to the number of bytes encoded
;	                    coded in the <address>, <data> & <checksum> fields
;
;	    address     4   S0: always "0000"
;	                    S1: record load address
;	                    S5: number of transmitted S1 records (64K max)
;	                    S9: start address for code (optional)
;
;	    data        *   S0: descriptive information (optional)
;	                *   S1: data to be loaded into RAM, starting at the add-
;	                        ress in the <address> field
;	                0   S5: not used
;	                0   S9: not used
;
;	    checksum    2   8 bit ones complement computed from the <length>,
;	                    <address> & <data> fields
;	    ----------------------------------
;
;	Sizes are in ASCII characters.  A size indicated by * will vary depend-
;	ing on the data in the field.  Values in the <address> field are in big-
;	endian format.  Hence "C4" followed by "21" means the binary value $C421
;	or decimal 50209.
;
;	This function will attempt to flush all input from the auxiliary TIA-232
;	channel prior to commencing the load.  A data stream should not be sent
;	until the "Ready" prompt is displayed.
;	
;
monlsr   bcc .0000010          ;bank entered
;
         lda reg_pbx           ;use current bank...
         bra .0000040          ;as default
;
.0000010 jsr facasize          ;check arg size
         cmp #s_word
         bcc .0000030          ;bank in range
;
.0000020 jmp monerr            ;abort w/error
;
.0000030 lda faca              ;entered bank
;
.0000040 sta loadbank          ;set load bank
         stz pageoff           ;reset page offset
         jsr getparm           ;check for page offset
         bcs .0000050          ;none entered
;
         jsr facasize          ;check size
         cmp #s_word
         bcs .0000020          ;out of range
;
         lda faca              ;entered page offset
         sta pageoff           ;save it
         jsr getparm           ;should be no more input
         bcc .0000020          ;error
;
.0000050 longa
         stz nrecs             ;reset S1 record count
         shorta
         jsr enabchb           ;enable channel B receiver
         ldx #20
;
.0000060 wai                   ;flush...
         jsr clrchanb          ;channel B
         dex
         bne .0000060
;
         pea #lr_rtl           ;"Ready"
         jsr sprint            ;print string
;
;	
;	MAIN PROCESSING LOOP
;	
;
monlsr01 longa
         stz cksumcmp          ;reset computed checksum
         shortr
         ldx #0                ;buffer index
         ldy #s_ibuf           ;input counter
;
;
;	acquire record...
;
.0000010 jsr getchanb          ;get datum from channel B
         bcc .0000020          ;gotten
;
         jmp monlsr07          ;aborted by user
;
.0000020 beq monlsr02          ;EOL, process record
;
         sta ibuffer,x         ;store datum
         inx                   ;bump index
         dey                   ;decrement counter
         bne .0000010          ;buffer not full, get next
;
         pea #lr_bufor          ;"buffer overrun..."
         jmp monlsr06          ;abort w/error
;
monlsr02 stz ibuffer,x         ;terminate input
         stz ibufidx           ;reset buffer index
         lda nrecs             ;total S1 records LSB
         ora nrecs+s_byte      ;total S1 records MSB
         bne .0000010          ;not 1st record
;
         pea #lr_load           ;"loading..."
         jsr sprint
;
;
;	evaluate record preamble...
;
.0000010 jsr getchar           ;get 1st char from record
         cmp #srecpre          ;preamble char
         bne monlsr03          ;error
;
         jsr getchar           ;get record type
         beq monlsr03          ;unexpected end  error
;
         sec
         sbc #'0'              ;change to binary
         cmp #m_rectyp+1       ;check range
         bcs monlsr03          ;out of range  error
;
         sta rectype           ;set record type
;
;
;	evaluate record length...
;
         jsr getcvt            ;get & convert length
         bcs monlsr03          ;error
;
         tax
         .rept 2               ;deduct address length
           dex
         .endr
         stx reclen            ;save record length
         jsr acksum            ;add to checksum
;
;
;	evaluate load address...
;
         jsr getcvt            ;get & convert address MSB
         bcs monlsr03          ;error
;
         sta loadaddr+s_byte   ;save load address MSB
         jsr acksum            ;add to checksum
         jsr getcvt            ;get & convert address LSB
         bcs monlsr03          ;error
;
         sta loadaddr          ;save load address LSB
         jsr acksum            ;add to checksum
;
;
;	evaluate data field...
;
         ldx #0                ;rebuffer index
         ldy reclen            ;record length
;
.0000020 jsr getcvt            ;get & convert
         bcs monlsr03          ;premature record end
;
         dey                   ;len -= 1
         beq monlsr04          ;.A = checksum byte
;
         sta ibuffer,x         ;store converted byte
         jsr acksum            ;add to checksum
         inx
         bra .0000020          ;get next
;
monlsr03 pea #lr_corup          ;"corrupted record..."
         jmp monlsr06
;
monlsr04 sta datax             ;save record checksum
         lda cksumcmp          ;running checksum
         eor #%11111111        ;ones complement
         cmp datax             ;check against record checksum
         bne monlsr03          ;bad checksum  error
;
         lda rectype           ;record type
         asl                   ;generate index
         tax
         jmp (lsrextab,x)      ;process record type
;
;
;	process S0 record...
;
;	
;	S0 data segment is not evaluated.
;	
;
monlsrs0 bra monlsr05          ;get next record
;
;
;	process S1 record...
;
monlsrs1 jsr adjloadr          ;adjust load address
         longa
         lda nrecs             ;any records processed?
         bne .0000010          ;yes
;
         lda loadaddr          ;load address
         sta strtaddr          ;also start address
;
.0000010 shortr
         dec reclen            ;bytes to copy
         ldx #0                ;buffer index
;
.0000020 lda ibuffer,x         ;get byte &...
         sta [loadaddr]        ;write to RAM
         dec reclen            ;all bytes copied?
         beq .0000030          ;yes
;
         longa                 ;no
         inc loadaddr          ;bump load address
         shorta
         inx                   ;bump index
         bne .0000020          ;next
;
.0000030 longa
         lda loadaddr
         sta endaddr           ;set working end address
         inc nrecs             ;bump processed record count
         shorta
         lda #'.'              ;progress indicator
         jsr putcha            ;print on console
;
monlsr05 shortr
         jmp monlsr01          ;get next record
;
;
;	process S5 record...
;
monlsrs5 dec reclen            ;data field should be empty
         bne monlsr03          ;it isn't  error
;
         longa
         lda loadaddr          ;expected S1 record count
         cmp nrecs             ;processed S1 record count
         beq monlsr05          ;count okay  continue
;
         pea #lr_recct          ;"invalid S1 count..."
         bra monlsr06          ;abort w/error
;
;
;	process S9 record...
;
monlsrs9 dec reclen            ;data field should be empty
         bne monlsr03          ;it isn't  error
;
         longa
         lda nrecs             ;total S1 records
         bne .0000010          ;at least 1 record loaded
;
         pea #lr_norec          ;"no records..."
         bra monlsr06          ;abort w/error
;
.0000010 shorta
         lda loadbank          ;load bank
         sta reg_pbx           ;default for execution
         jsr adjloadr          ;adjust load address
         longa
         lda loadaddr          ;load address from S9
         bne .0000020          ;valid
;
         lda strtaddr          ;initial load address
;
.0000020 sta reg_pcx           ;set execution address
         shorta
         pea #lr_addrs          ;"start address..."
         jsr sprint
         jsr dpypbr            ;display bank
         longa
         lda strtaddr          ;initial load address
         shorta
         jsr dpyhexw           ;display start address
         pea #lr_addre          ;"end address"
         jsr sprint
         jsr dpypbr            ;display bank
         longa
         lda endaddr           ;final load address
         shorta
         jsr dpyhexw           ;display end address
         bra monlsr07          ;back to monitor executive
;
;
;	undefined record type error handler...
;
monlsrut pea #lr_type           ;"invalid..."
;
;
;	general error processing...
;
monlsr06 shortr
         jsr sprint            ;print error message
;
;
;	terminate S-record processing...
;
monlsr07 jsr disabchb          ;disable channel B port
         jmp monce             ;back to monitor executive
;
	.end
