;================================================================================
;
;MACHINE LANGUAGE MONITOR: ISSUE SCSI COMMAND
;
;	
;	syntax: ! <command>, where <command> is:
;
;	        B <ID>,<LUN>                   rewind tape
;	        F <ID>,<LUN>                   format unit
;	        I                              reset SCSI subsystem
;	        R <ID>,<LUN>,<LBA>,<N>,<ADDR>  read from device
;	        S <ID>,<LUN>,<ADDR>            request sense data
;	        W <ID>,<LUN>,<LBA>,<N>,<ADDR>  write to device
;
;	Parameters:
;
;	        <ID>   SCSI ID, $00-$07
;	        <LUN>  logical unit, $00-$07
;	        <LBA>  logical block address, e.g., $1F410C
;	        <N>    quantity of data to read or write ($0000-$FFFF)
;	        <ADDR> buffer starting address
;
;	NOTES: 1) Commands are not case-sensitive.  Whitespace may used between
;	          the ! prefix & the command letter, & between the command let-
;	          ter & 1st parameter.
;	       2) Whitespace may substituted for commas to delimit parameters.
;	       3) As with all other monitor commands, parameters are assummed to
;	          be hex unless prefixed with an alternate radix.
;	       4) <N> is blocks when accessing a block device, e.g., a disk or
;	          is bytes when accessing a stream device, e.g., a tape.
;	       5) <N> & <ADDR> are not checked for validity.  Improper values
;	          may result in overwrite of critical system areas & cause fat-
;	          ality.
;
;	In some cases, the monitor may report an error that was actually genera-
;	ted within the SCSI subsystem driver, such as device not present.  Such
;	errors are currently not decoded & displayed.
;	
;
;================================================================================
;
;MACHINE LANGUAGE MONITOR: RECALIBRATE DISK/REWIND TAPE
;
;	
;	syntax: !b <ID> <LUN>
;	
;
scsibom  bcs scsierrc          ;required parameters missing
;
         jsr getidlun          ;get SCSI ID & LUN
         bcs scsierrc          ;missing LUN or out of range
;
;	
;	GETIDLUN exits with the LUN in .X if successful.
;	
;
         lda #sc_recal         ;SCSI opcode
         bra scsicoma          ;execute command
;
;	
;	SC_RECAL has been declared  obsolete for block devices.   Hence
;	the effect is undefined if the target  is not a stream  device.
;	Disks will either interpret it as a recalibrate command or will
;	generate "check condition," the latter which will result in the
;	monitor flagging an error.
;	
;
;================================================================================
;
;MACHINE LANGUAGE MONITOR: FORMAT UNIT/ERASE TAPE
;
;	
;	syntax: !f <ID> <LUN>
;	
;
scsifmt  bcs scsierrc          ;required parameters missing
;
         jsr getidlun          ;get SCSI ID & LUN
         bcs scsierrc          ;missing LUN or out of range
;
         bit flimflag          ;device type
         bpl .0000010          ;block
;
         lda #sc_erase         ;stream: erase
         .byte bitabs
;
.0000010 lda #sc_fmt           ;block: format
         ldx #0                ;wait 'til done
;
scsicoma jsr setupcdb          ;initialize CDB
         jsr getparm           ;check for excess input
         bcc scsierrc          ;error
;
         longx
         lda operand           ;target ID
         ldx !#auxbuf          ;CDB address
         jsr scsicmd           ;execute command
         bcc scsixcom          ;executed
;
         bra scsierrc          ;error
;
;================================================================================
;
;MACHINE LANGUAGE MONITOR: RESET SCSI SUBSYSTEM
;
;	
;	syntax: !i
;	
;
scsirst  bcc scsierrc          ;no parameters allowed
;
         pea #dc_cs
         jsr sprint            ;clear screen
         jsr scsiinit          ;initialize & reset
;
scsixcom jmp monce             ;back to command exec
;
scsierrc jmp monerr            ;goto error handler
;
;================================================================================
;
;MACHINE LANGUAGE MONITOR: READ FROM DEVICE
;
;	
;	syntax: !r <ID> <LUN> <LBA> <N> <ADDR>
;	
;
scsiread bcs scsierrc          ;no parameters given
;
         jsr getidlun          ;get SCSI ID & LUN
         bcs scsierrc          ;out of range
;
         lda #sc_read          ;basic read opcode
         bra scsirwc           ;perform read operation
;
;================================================================================
;
;MACHINE LANGUAGE MONITOR: REQUEST SENSE DATA
;
;	
;	syntax: !s <ID> <LUN> <ADDR>
;	
;
scsisens bcs scsierrc          ;no parameters given
;
         jsr getidlun          ;get SCSI ID & LUN
         bcs scsierrc          ;out of range
;
         lda #sc_getsd         ;request sense opcode
         jsr setupcdb          ;initialize CDB
         lda #s_reqsen         ;data size
         sta auxbuf+so_xfrl    ;set it
         jsr setuprAA          ;set up buffer address
         bcs scsierrc          ;error
;
         longx
         lda operand           ;target ID
         ldx !#auxbuf          ;CDB address
         ldy addra             ;buffer address
         jsr scsicmd           ;execute command
         bcc scsixcom          ;executed
;
         bra scsierrc          ;error
;
;================================================================================
;
;MACHINE LANGUAGE MONITOR: WRITE TO DEVICE
;
;	
;	syntax: !w <ID> <LUN> <LBA> <N> <ADDR>
;	
;
scsiwrit bcs scsierrc          ;no parameters given
;
         jsr getidlun          ;get SCSI ID & LUN
         bcs scsierrc          ;out of range
;
         tay                   ;protect LUN
         lda #sc_writ          ;basic write opcode
;
;
;	read/write common code...
;
scsirwc  bit flimflag          ;addressing capability
         bvc .0000010          ;21 bit
;
         ora #sg2mask          ;set group 2 bit
;
.0000010 jsr setupcdb          ;initialize CDB
         jsr setuprwb          ;set up access params
         bcs scsierrc          ;error
;
         longx
         lda operand           ;target ID
         ldx !#auxbuf          ;CDB address
         ldy addra             ;buffer address
         jsr scsicmd           ;execute command
         bcc scsixcom          ;executed
;
         bra scsierrc          ;error
;
	.end
