;===============================================================================
;
;scsicmd: EXECUTE SCSI COMMAND
;
;	
;	Preparatory Ops : .A:  8 bits: target device SCSI ID
;	                  .X: 16 bits: command descriptor block address
;	                  .Y: 16 bits: buffer address (1)
;
;	Register Returns: .A: exit code (2,3)
;	                  .B: $00
;	                  .X: used
;	                  .Y: used
;	                  DB: entry value
;	                  DP: entry value
;	                  PB: entry value
;	                 
;	MPU Flags: NVmxDIZC
;	           ||||||||
;	           |||||||+> 0: okay (2)
;	           |||||||      1: exception (3)
;	           ||||||+> reflects exit code
;	           ||||++> 0
;	           ||++> 1
;	           ++> undefined
;
;	Example:
;
;	Notes: 1) .Y is "don't care" if the command doesn't require access to a
;	          buffer.
;
;	       2) If no error exists $00 will be returned in .A.
;
;	       3) If an exception is flagged one of the following error codes
;	          will be returned in .A:
;
;	              ED_DEVID  invalid SCSI ID (range)
;	              ED_DEVNE  device not enumerated
;	              ED_USFUN  unsupported driver function
;	              ED_USGRP  unsupported command group
;	              ED_USPHS  unsupported bus phase
;	              ED_DEVNP  device not present
;	              ED_ABORT  aborted transaction
;	              ED_CHECK  check condition
;	              ED_CNTRL  general controller error
;	              ED_FIFO   FIFO count error
;	              ED_ILCMD  illegal controller command
;	              ED_RESET  unexpected bus reset
;
;	              Also returned if HBA was not detected during SCSI subsys-
;	               tem initialization.
;
;	          The above are defined in include_scsi/keys_driver.asm.  DO NOT
;	          hard-code any of these definitions into any program.  Use the
;	          INCLUDE file to define them.
;	
;
scsicmd  rep #%11111111        ;clear all SR flags
         phd                   ;save current DP
         phy                   ;save buffer address
         phx                   ;save CDB address
;
;
sccdbptr .= 1                  ;CDB address
scbufptr .= sccdbptr+s_ptr     ;buffer address
reg_dp   .= scbufptr+s_ptr     ;DP
scretadr .= reg_dp+s_word      ;return address
;
;
         pea #kerneldp
         pld                   ;select kernel direct page
         stz ivscsi            ;clear alternate vector
         shortr
         bit scsihba           ;host adapter present?
         bpl ex_cntrl          ;no, abort
;
         jsr sslotptr          ;point at device table slot
         bcs ex_devid          ;invalid target ID  error
;
         tax                   ;protect target ID
         ldy #so_opcod         ;CDB opcode offset
         lda (sccdbptr,s),y    ;get opcode
         cmp #sc_inqry         ;inquiry?
         beq .0000010          ;yes, skip enumeration check
;
         cmp #sc_getsd         ;request sense?
         beq .0000010          ;yes, skip enumeration check
;
         cmp #sc_tur           ;test unit ready?
         beq .0000010          ;yes, skip enumeration check
;
         xba                   ;protect opcode
         lda (scsididx)        ;check device status
         bpl ex_devne          ;not enumerated  error
;
         xba                   ;restore opcode
;
;
;	check command group validity...
;
.0000010 and #grpmask          ;isolate command group
         ldy #n_scgrps-1
;
.0000030 cmp scgroups,y        ;find the group
         beq .0000040          ;found it
;
         dey
         bpl .0000030          ;try another
;
         bra ex_usgrp          ;invalid group  error
;
.0000040 sty scsigidx          ;save group index
;
;
;	set up controller & initiate selection...
;
         lda #sccfifc
         sta io_scsi+sr_cmd    ;clear FIFO
         stx io_scsi+sr_sdid   ;set target ID
         ldx scsizes,y         ;use CDB size as counter
         ldy #0                ;CDB index
;
.0000050 lda (sccdbptr,s),y    ;load CDB...
         sta io_scsi+sr_fifo   ;into FIFO
         iny                   ;bump source index
         dex                   ;step counter
         bne .0000050          ;next
;
         lda #sccsel           ;select wo/ATN command
;
;= = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
;
;c94cmd: ISSUE COMMAND TO SCSI CONTROLLER
;
;	
;	Upon entry the 8 bit controller command code must be in .A &
;	any other controller setup steps, such as setting the target
;	ID or priming the FIFO, must have been completed.
;	
;
c94cmd   sta io_scsi+sr_cmd    ;issue controller command
;
.0000010 wai                   ;wait for controller...
         bra .0000010          ;response...
;
;	
;	There's a possibility of deadlock here should the controller fail to
;	generate an IRQ after be issued the command.  The above loop will be
;	terminated when the IRQ comes, as the SCSI ISR will revector the MPU
;	to SCSICMDA or to an alternate location specified in IVSCSI.  Should
;	that IRQ never come...well, that's why there's a reset button.
;	
;
;= = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
;
;phinvald: PROCESS INVALID BUS PHASES
;
phinvald jsr scsirstb          ;force a bus reset
         bra ex_usphs          ;invalid phase error
;
;= = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
;
;grosserr: SCSI HARDWARE FAILURE HANDLER
;
grosserr jsr scsirstb          ;force a bus reset
;
;= = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
;
;ERROR EXIT HANDLERS
;
ex_cntrl lda #ed_cntrl         ;HBA not present/hardware failure
         .byte bitabs
ex_abort lda #ed_abort         ;transaction aborted
         .byte bitabs
ex_check lda #ed_check         ;check condition
         .byte bitabs
ex_devid lda #ed_devid         ;invalid SCSI ID (range)
         .byte bitabs
ex_devne lda #ed_devne         ;device not enumerated
         .byte bitabs
ex_devnp lda #ed_devnp         ;device not present
         .byte bitabs
ex_ilcmd lda #ed_ilcmd         ;illegal controller command
         .byte bitabs
ex_reset lda #ed_cntrl         ;unexpected bus reset
         .byte bitabs
ex_usfun lda #ed_usfun         ;unsupported driver function
         .byte bitabs
ex_usgrp lda #ed_usgrp         ;unsupported command group
         .byte bitabs
ex_usphs lda #ed_usphs         ;unsupported bus phase
;
         sec                   ;flag an error &...
         bra ex_comm           ;exit
;
;= = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
;
;COMMON DRIVER EXIT
;
ex_okay  lda #ed_okay          ;okay exit code
         clc                   ;no error
;
ex_comm  longr
         plx                   ;clean up stack
         ply
         pld
         and !#%11111111       ;clear .B/condition .Z
         shortr
         rts
;
;= = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
;
;scsicmda: CONTROLLER COMMAND COMPLETION ANALYSIS
;
;	
;	Upon entry, the following 8 bit controller register values
;	are expected:
;
;	    .A: interrupt status:
;
;	        xxxxxxxx
;	        ||||||||
;	        |||||||+> 1: selected
;	        ||||||+> 1: selected w/attention
;	        |||||+> 1: reselected
;	        ||||+> 1: successful operation
;	        |||+> 1: service request
;	        ||+> 1: disconnected
;	        |+> 1: invalid command
;	        +> 1: bus reset
;
;	    .X: command state
;
;	        0000xxxx
;	            ||||
;	            |+++> internal state
;	            +> 0: sync offset register at maximum
;
;	    .Y: general status
;
;	        xxxxxxxx
;	        ||||||||
;	        |||||+++> current bus phase
;	        ||||+> 1: valid group code
;	        |||+> 1: transfer count = zero
;	        ||+> 1: bus parity error
;	        |+> 1: gross error
;	        +> 1: interrupt pending
;	
;
scsicmda bit #c94resel | c94sela | c94sel
         bne ex_usfun          ;function not supported
;
         asl                   ;bus reset?
         bcs ex_reset          ;yes  error
;
         asl                   ;illegal controller command?
         bcs ex_ilcmd          ;yes  error
;
         asl                   ;disconnected?
         bcs .0000010          ;yes, analyze cause
;
         asl                   ;bus service request?
         bcc .0000010          ;no, transaction completed
;
         tya                   ;general status
         and #c94phase         ;isolate the bus phase
         asl                   ;convert phase to...
         tax                   ;table index &...
         jmp (scsiptab,x)      ;execute function
;
;
;	process bus disconnection...
;
.0000010 lda io_scsi+sr_cmd    ;was last controller...
         cmp #sccsel           ;command a select?
         beq ex_devnp          ;yes, device not present
;
         txa                   ;get command status
         and #c94isrst         ;isolate state bits
         cmp #c94issqc         ;command steps completed?
         bne ex_cntrl          ;no  controller error
;
;
;	execute function completed steps...
;
         longa
         lda !#.0000020        ;where to return
         sta ivscsi            ;set up vector
         shorta
         lda #sccmsga          ;message accepted
         bra c94cmd            ;initiate command...
;
;	
;	For now, we're not analyzing the message.  There's just
;	not enough room in the ROM for the required code & data.
;	
;
.0000020 lda scsistat          ;last status
         beq ex_okay           ;okay
;
         cmp #ss_cmet          ;condition met?
         beq ex_okay           ;yes
;
         cmp #ss_check         ;check condition?
         beq ex_check          ;yes
;
         bra ex_abort          ;other status
;
;= = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
;
;phmesout: PROCESS MESSAGE-OUT PHASE
;
phmesout longx
         lda #s_mesg           ;data size
         ldx !#scsimesg        ;where to get it
         jsr dmaxfrit          ;do DMA transfer setup
         bra ssxfrout          ;goto DMA output function
;
;= = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
;
;phmesin: PROCESS MESSAGE-IN PHASE
;
phmesin  longx
         lda #s_mesg           ;data size
         ldx !#scsimesg        ;where to put it
         jsr dmaxfrit          ;do DMA transfer setup
         bra ssxfrin           ;goto DMA input function
;
;= = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
;
;phstatus: PROCESS STATUS PHASE
;
phstatus longx
         lda #s_stat           ;data size
         ldx !#scsistat        ;where to put it
         jsr dmaxfrit          ;do DMA transfer setup
         bra ssxfrin           ;goto DMA input function
;
;= = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
;
;phdatout: PROCESS DATA OUT PHASE
;
phdatout jsr dmaxfrsu          ;do data phase setup
;
;= = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
;
;ssxfrout: DMA OUTPUT TRANSFER
;
;	
;	Preparatory Ops : .X: 16 bit buffer pointer
;
;	Register Returns: none
;	
;
ssxfrout lda mm_ram,x          ;get datum
         inx                   ;next buffer location
;         
.0000010 bit io_scsiq+sr_fifo  ;FIFO full?
         bpl .0000010          ;yes
;
         sta io_scsid+sr_fifo  ;no, store datum
         bra ssxfrout          ;repeat
;
;= = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
;
;phdatin: PROCESS DATA IN PHASE
;
phdatin  jsr dmaxfrsu          ;data phase setup
;
;= = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
;
;ssxfrin: DMA INPUT TRANSFER
;
;	
;	Preparatory Ops : .X: 16 bit buffer pointer
;
;	Register Returns: none
;	
;
ssxfrin  bit io_scsiq+sr_fifo  ;FIFO empty?
         bpl ssxfrin           ;yes, keep trying
;
         lda io_scsid+sr_fifo  ;get &...
         sta mm_ram,x          ;store datum
         inx                   ;next buffer location
         bra ssxfrin           ;repeat
;
;= = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
;
;dmaxfrsu: DMA TRANSFER SETUP
;
dmaxfrsu shortr                ;8 bit everything
;
;
.retadr  .set 1                ;return address
.cdbptr  .set .retadr+s_ptr    ;CDB address
.bufptr  .set .cdbptr+s_ptr    ;buffer address
;
;
         ldy #o_sdtype
         lda (scsididx),y      ;get device flag
         and #sdtmask          ;extract device type
         cmp #sd_da            ;direct access?
         beq .0000010          ;yes
;
         cmp #sd_cdrom         ;CD or DVD?
         bne .0000030          ;no
;
.0000010 ldy #so_opcod         ;CDB opcode offset
         lda (.cdbptr,s),y     ;get opcode from CDB
         ldx #sctloset-sccsttab-1
;
.0000020 cmp sccsttab,x        ;compare to table
         beq .0000040          ;translate transfer size
;
         dex
         bpl .0000020          ;try next
;
.0000030 clc                   ;don't translate size
;
.0000040 lda #0
         xba                   ;initialize .B
         ldx scsigidx          ;get group index
         ldy sctloset,x        ;get transfer size field offset
         lda scctztab,x        ;get transfer size field size
         tax
         lda (.cdbptr,s),y     ;get transfer size
         dex                   ;byte or word?
         beq .0000050          ;byte, hence no MSB
;
         xba                   ;transfer size MSB
         iny                   ;point at & get...
         lda (.cdbptr,s),y     ;transfer size LSB
;
.0000050 bcc .0000070          ;no size conversion needed
;
         longr
         tax                   ;use transfer size as counter
         lda !#0               ;start with no bytes
         ldy !#o_sdbsiz        ;block size index
         clc
;
.0000060 adc (scsididx),y      ;bytes = bytes + block size
         dex                   ;step counter
         bne .0000060          ;more blocks
;
.0000070 longr
         tay                   ;protect count
         lda .bufptr,s         ;get buffer address
         tax                   ;now RAM index
         tya                   ;get count & fall thru
         shorta
;
;= = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
;
;dmaxfrit: INITIATE A DMA TRANSFER
;
;	
;	Preparatory Ops : .C: 16 bits: transfer count
;
;	Register Returns: .A: used
;	                  .B: used
;	                  .X: entry value
;	                  .Y: entry value
;	                 
;	MPU Flags: NVmxDIZC
;	           ||||||||
;	           ||||||++> undefined
;	           |||||+> 1
;	           ||||+> 0
;	           |||+> 0
;	           ||+> 1
;	           ++> undefined
;	
;
dmaxfrit sta io_scsi+sr_stcl   ;set transfer count LSB
         xba
         sta io_scsi+sr_stch   ;set transfer count MSB
;
;	
;	NOTE: A 16 bit write to the 'C94 transfer count register
;	      doesn't work due to yet-to-be-resolved hardware
;	      timing issues.
;	
;
         lda #sccfifc
         sta io_scsi+sr_cmd    ;flush FIFO
         lda #0
         xba                   ;clear .B
         lda #sccinfod         ;DMA info xfer command
         sta io_scsi+sr_cmd    ;initiate transfer
         rts
;
;===============================================================================
;
;scsibrst: RESET SCSI BUS w/DEVICE RECOVERY DELAY
;
scsibrst shorta
         jsr scsirstb
         lda #0
         xba
         lda #scsirdly         ;recovery period
         jmp utdelay           ;delay
;
;===============================================================================
;
;scsihaid: REPORT HBA BUS ID
;
scsihaid shorta
         lda io_scsi+sr_cr1    ;'CF94 control register 1
         and #haidmask         ;isolate bus ID bits
         rts
;
;===============================================================================
;
;scsirstb: RESET SCSI BUS
;
scsirstb lda #sccrstb          ;bus reset command
         sta io_scsi+sr_cmd    ;issue to controller
         rts
;
	.end
